@@ -29,23 +29,25 @@ module Control.Monad.Trans.Control
     ( -- * MonadTransControl
       MonadTransControl(..), Run
 
-      -- ** Defaults for MonadTransControl
+      -- ** Defaults
       -- $MonadTransControlDefaults
     , RunDefault, defaultLiftWith, defaultRestoreT
 
       -- * MonadBaseControl
     , MonadBaseControl (..), RunInBase
 
-      -- ** Defaults for MonadBaseControl
+      -- ** Defaults
       -- $MonadBaseControlDefaults
     , ComposeSt, RunInBaseDefault, defaultLiftBaseWith, defaultRestoreM
 
       -- * Utility functions
-    , control, embed, embed_
+    , control, embed, embed_, captureT, captureM
 
     , liftBaseOp, liftBaseOp_
 
     , liftBaseDiscard, liftBaseOpDiscard
+
+    , liftThrough
     ) where
 
 
@@ -464,6 +466,16 @@ embed_ :: MonadBaseControl b m => (a -> m ()) -> m (a -> b ())
 embed_ f = liftBaseWith $ \runInBase -> return (void . runInBase . f)
 {-# INLINABLE embed_ #-}
 
+-- | Capture the current state of a transformer
+captureT :: (MonadTransControl t, Monad (t m), Monad m) => t m (StT t ())
+captureT = liftWith $ \runInM -> runInM (return ())
+{-# INLINABLE captureT #-}
+
+-- | Capture the current state above the base monad
+captureM :: MonadBaseControl b m => m (StM m ())
+captureM = liftBaseWith $ \runInBase -> runInBase (return ())
+{-# INLINABLE captureM #-}
+
 -- | @liftBaseOp@ is a particular application of 'liftBaseWith' that allows
 -- lifting control operations of type:
 --
@@ -525,3 +537,13 @@ liftBaseOpDiscard :: MonadBaseControl b m
                   ->  (a -> m ()) -> m c
 liftBaseOpDiscard f g = liftBaseWith $ \runInBase -> f $ void . runInBase . g
 {-# INLINABLE liftBaseOpDiscard #-}
+
+-- | Transform an action in @t m@ using a transformer that operates on the underlying monad @m@
+liftThrough
+    :: (MonadTransControl t, Monad (t m), Monad m)
+    => (m (StT t a) -> m (StT t b)) -- ^
+    -> t m a -> t m b
+liftThrough f t = do
+  st <- liftWith $ \run -> do
+    f $ run t
+  restoreT $ return st
